#!/usr/bin/env node
import { green, red } from "colorette";
import yargs from "yargs";
import { hideBin } from "yargs/helpers";
import ConfigHelper from "./libs/ConfigHelper.js";
import Logger from "./libs/Logger.js";
import { CopyTask } from "./task/CopyTask.js";
import { HelperTask } from "./task/HelperTask.js";
import { IsomorphicPack } from "./task/IsomorphicPack.js";
import { ModuleFederatePack } from "./task/ModuleFederatePack.js";
import { PackageInfo } from "./task/PackageInfo.js";
import { PublishTask } from "./task/PublishTask.js";
import { ServerApiPack } from "./task/ServerApiPack.js";
import { ServerPack } from "./task/ServerPack.js";
import { StandalonePack } from "./task/StandalonePack.js";
const log = Logger("ProjectBuild");
export class ProjectBuild {
    isDebugMode = false;
    isWatchMode = false;
    isRunServerMode = false;
    isPublishMode = false;
    isAnalyzMode = false;
    isOnlyStandalone = false;
    isHotReload = false;
    async start() {
        try {
            await this.build();
            if (this.isPublishMode) {
                await this.publish();
            }
        }
        catch (e) {
            log.error(red("ProjectBuild error: "), e);
        }
    }
    setAnalyzMode(isAnalyzMode) {
        this.isAnalyzMode = isAnalyzMode;
    }
    setDebugMode(isDebugMode) {
        this.isDebugMode = isDebugMode;
    }
    setWatchMode(isWatchMode) {
        this.isWatchMode = isWatchMode;
    }
    setPublishModel(isPublishMode) {
        this.isPublishMode = isPublishMode;
    }
    setRunServerMode(isRunServerMode) {
        this.isRunServerMode = isRunServerMode;
    }
    setOnlyStandalone(isOnlyStandalone) {
        this.isOnlyStandalone = isOnlyStandalone;
    }
    setHotReloadMode(isHotReload) {
        this.isHotReload = isHotReload;
    }
    async publish() {
        await new PublishTask().run();
    }
    async build() {
        const task = new HelperTask();
        task.setDebugMode(this.isDebugMode);
        task.start();
        try {
            await task.cleanAsync();
            if (!this.isOnlyStandalone) {
                const packageInfo = new PackageInfo();
                packageInfo
                    .setDebugMode(this.isDebugMode)
                    .setWatchMode(this.isWatchMode);
                await packageInfo.run();
                const copyTask = new CopyTask("./config", "./config");
                copyTask.setDebugMode(this.isDebugMode);
                await copyTask.run();
                const isomorphicClientPack = new IsomorphicPack();
                isomorphicClientPack.setHotReloadMode(this.isHotReload);
                isomorphicClientPack
                    .setDebugMode(this.isDebugMode)
                    .setWatchMode(this.isWatchMode)
                    .setAnalyzMode(this.isAnalyzMode);
                await isomorphicClientPack.run();
                const shouldServerApiBuild = ConfigHelper.get("serverapi", false);
                if (shouldServerApiBuild) {
                    const serverApiPack = new ServerApiPack();
                    serverApiPack
                        .setWatchMode(this.isWatchMode)
                        .setDebugMode(this.isDebugMode);
                    await serverApiPack.run();
                }
                const serverPack = new ServerPack();
                serverPack.setHotReloadMode(this.isHotReload);
                serverPack
                    .setAutoRun(this.isRunServerMode)
                    .setDebugMode(this.isDebugMode)
                    .setWatchMode(this.isWatchMode);
                await serverPack.run();
                const shouldModuleFederateBuild = ConfigHelper.get("federation", false);
                if (shouldModuleFederateBuild && shouldModuleFederateBuild.exposes) {
                    const moduleFederatePack = new ModuleFederatePack();
                    moduleFederatePack
                        .setDebugMode(this.isDebugMode)
                        .setWatchMode(this.isWatchMode);
                    await moduleFederatePack.run();
                }
            }
            const shouldStandaloneBuild = ConfigHelper.get("standalone", false);
            if (shouldStandaloneBuild) {
                const standalonePack = new StandalonePack();
                standalonePack
                    .setDebugMode(this.isDebugMode)
                    .setWatchMode(this.isWatchMode);
                await standalonePack.run();
            }
            else if (this.isOnlyStandalone) {
                throw new Error(`the value of 'standalone' field must be an object in ./config/configure.json while build with --ost argument`);
            }
            console.log(green("build success"));
        }
        catch (e) {
            log.error(red("ProjectBuild has error: "), e);
        }
        task.end();
    }
}
(async () => {
    const projectBuild = new ProjectBuild();
    const argv = yargs(hideBin(process.argv)).argv;
    if (argv.debug) {
        projectBuild.setDebugMode(true);
    }
    if (argv.watch) {
        projectBuild.setWatchMode(true);
    }
    if (argv.runServer) {
        projectBuild.setRunServerMode(true);
    }
    if (argv.hotReload) {
        projectBuild.setHotReloadMode(true);
    }
    if (argv.analyz) {
        projectBuild.setAnalyzMode(true);
    }
    if (argv.onlystandalone) {
        projectBuild.setOnlyStandalone(true);
    }
    try {
        await projectBuild.start();
    }
    catch (error) {
        process.exit(1);
    }
})();
