import { bold, cyan, red, white, yellow } from "colorette";
import fs from "fs-extra";
import path from "path";
import webpack from "webpack";
import { merge } from "webpack-merge";
import WebpackDevServer from "webpack-dev-server";
import yargs from "yargs";
import { hideBin } from "yargs/helpers";
import clientBase from "../config/client.base.js";
import serverBase from "../config/server.base.js";
import HelperTask from "../task/HelperTask.js";
import Logger from "./Logger.js";
import TaskBase from "./TaskBase.js";
const log = Logger();
const argv = yargs(hideBin(process.argv)).argv;
const hooksName = "alcor-webpack-task-base";
const taskCuzConfFileNameMap = {
    "IsomorphicPack": "client",
    "ModuleFederatePack": "mf",
    "StandalonePack": "standalone",
    "ServerApiPack": "api",
    "ServerPack": "server",
};
export class WebpackTaskBase extends TaskBase {
    helperTask;
    static compileQueue = [];
    static compileDoneCallback = [];
    index = 0;
    isHotReload = false;
    constructor(taskName = "WebpackTaskBase") {
        super(taskName);
        this.helperTask = new HelperTask();
        this.index = WebpackTaskBase.compileQueue.length;
        WebpackTaskBase.compileQueue.push({
            state: false,
            hash: "",
        });
    }
    compileInvalid(fileName, changeTime) {
        const context = WebpackTaskBase.compileQueue[this.index];
        if (context) {
            context.state = false;
            log.info(`${cyan(this.taskName)}, file change: ${fileName}`, Date.now());
        }
    }
    compileWatchRun() {
        const context = WebpackTaskBase.compileQueue[this.index];
        if (context) {
            context.state = false;
        }
    }
    async compileDone(stats) {
        const context = WebpackTaskBase.compileQueue[this.index];
        if (context) {
            await this.done();
            context.state = true;
            context.hash = stats.hash;
        }
    }
    isAllCompileDone() {
        const result = WebpackTaskBase.compileQueue.every(context => context && context.state === true);
        return result;
    }
    printCompileResult(stats) {
        const info = stats.toJson();
        if (stats.hasErrors()) {
            this.helperTask.sendMessage(this.taskName, `代码有${info.errors.length}个错误`);
            info.errors.forEach(error => {
                log.error([
                    error.moduleName ? white("\n" + error.moduleName) : "",
                    red("\n  " + error.message),
                ].join(""));
                log.info(error);
            });
            if (this.isDebugMode === false) {
                const message = cyan(this.taskName) + ".fail";
                throw new Error(message);
            }
        }
        if (stats.hasWarnings()) {
            if (this.isDebugMode === true) {
                const showWarnList = info.warnings.filter(warning => {
                    return !(warning.moduleName
                        && /mizar\/server\/utils\/getApis\.js/.test(warning.moduleName)
                        && warning.message.indexOf('Critical dependency: the request of a dependency is an expression') > -1);
                });
                showWarnList.forEach(warning => {
                    log.warn([
                        warning.moduleName ? white("\n" + warning.moduleName) : "",
                        yellow("\n  " + warning.message),
                    ].join(""));
                    log.info(warning);
                });
                (info.warnings.length - showWarnList.length) > 0 && this.helperTask.sendMessage(this.taskName, "代码有警告");
            }
        }
        this.helperTask.sendMessage(this.taskName, "编译结束:" + this.count++);
        if (stats) {
            log.info(stats.toString({
                chunks: argv.verbose,
                colors: true
            }));
        }
    }
    compileFinishedCallback(fn) {
        WebpackTaskBase.compileDoneCallback.push(fn);
    }
    getCompileConfig(config) {
        const innerConf = merge((/server/i.test(this.taskName) ? serverBase : clientBase)(this.isDebugMode, this.isHotReload), config);
        return innerConf;
    }
    async compile(innerConf, isBaseConfigUsed = false) {
        let finalConf = isBaseConfigUsed ? innerConf : this.getCompileConfig(innerConf);
        const cuzConfigPath = path.resolve(`./webpack.config/${taskCuzConfFileNameMap[this.taskName]}.js`);
        if (fs.existsSync(cuzConfigPath)) {
            const cuzConf = (await import(cuzConfigPath)).default;
            if (typeof cuzConf === "function") {
                finalConf = merge(innerConf, cuzConf(innerConf));
            }
        }
        log.info("compile", { config: JSON.stringify(finalConf) });
        return new Promise((resolve, reject) => {
            const compiler = webpack(finalConf);
            compiler.hooks.invalid.tap(hooksName, (fileName, changeTime) => {
                this.compileInvalid(fileName, changeTime);
            });
            compiler.hooks.watchRun.tapAsync(hooksName, (compiler, callback = () => { }) => {
                this.compileWatchRun();
                callback();
            });
            compiler.hooks.done.tapAsync(hooksName, async (stats, callback = () => { }) => {
                log.info(cyan(this.taskName), "done, newhash: ", stats.hash, " , oldhash: ", WebpackTaskBase.compileQueue[this.index].hash);
                await this.compileDone(stats);
                callback();
            });
            compiler.close(error => {
                if (error) {
                    log.error(red(`${cyan(this.taskName)} compile close error: `), error);
                    return;
                }
                log.info(cyan(this.taskName), " compile closed.");
            });
            if (this.taskName === "IsomorphicPack" && this.isHotReload) {
                const wds = new WebpackDevServer({
                    devMiddleware: {
                        stats: false,
                    },
                    headers: {
                        "Access-Control-Allow-Origin": "*",
                    },
                    hot: true,
                    liveReload: true,
                    port: 9000,
                    static: "/static/client",
                    ...(finalConf.devServer || {})
                }, compiler);
                wds.start().then(resolve).catch(reject);
                return;
            }
            const compileCallback = async (error, stats) => {
                if (error) {
                    this.helperTask.sendMessage(this.taskName, "webpack执行出错");
                    log.error(cyan(this.taskName), ` > ${red(bold("webpack error:"))}`);
                    log.error(error.stack || error);
                    if (error.details) {
                        log.error(error.details);
                    }
                    reject(error);
                    throw new Error(cyan(this.taskName) + " 运行有错误");
                }
                this.printCompileResult(stats);
                if (this.isAllCompileDone()) {
                    while (WebpackTaskBase.compileDoneCallback.length) {
                        await WebpackTaskBase.compileDoneCallback.shift()();
                    }
                }
                resolve();
            };
            if (this.isWatchMode) {
                this.watcher = compiler.watch({
                    ignored: /[\\/]node_modules[\\/]|[\\/]dist[\\/]|\.d\.ts$|\.js\.map$|\.css\.map$/i,
                    aggregateTimeout: 600,
                }, compileCallback);
            }
            else {
                compiler.run(async (error, stats) => {
                    compileCallback(error, stats);
                });
            }
        });
    }
    getEnvDef() {
        return this.isDebugMode ? "development" : "production";
    }
}
export default WebpackTaskBase;
