import { cyan, green } from "colorette";
import path from "path";
import sharePlugin from "../config/share.plugin.js";
import ConfigHelper from "../libs/ConfigHelper.js";
import getGlobalConfig, { assetModuleFilename, devLocalIdentName, prodLocalIdentName, } from "../libs/getGlobalConfig.js";
import Logger from "../libs/Logger.js";
import { shouldSourceModuled, } from "../libs/Utils.js";
import { WebpackTaskBase } from "../libs/WebpackTaskBase.js";
import { HelperTask } from "./HelperTask.js";
import RunServer from "./RunServer.js";
const log = Logger("ServerPack");
let timeout;
export class ServerPack extends WebpackTaskBase {
    globalConfig;
    autoRun = false;
    debugPort = 0;
    constructor(taskName = "ServerPack") {
        super(taskName);
        this.globalConfig = getGlobalConfig();
        this.src = path.resolve("./src/server/index");
        this.dist = path.resolve(`${this.globalConfig.rootOutput}`);
    }
    getPlugins() {
        const plugins = [];
        plugins.push(...sharePlugin.remoteMfPlugin);
        return plugins;
    }
    getCssLoaders(isDebugMode, extraLoaders = []) {
        const loaders = [];
        let localIdentName = prodLocalIdentName;
        let sourceMap = false;
        if (isDebugMode) {
            localIdentName = devLocalIdentName;
            sourceMap = true;
        }
        return loaders.concat([
            {
                loader: "css-loader",
                options: {
                    importLoaders: 1 + extraLoaders.length,
                    sourceMap,
                    modules: {
                        auto: shouldSourceModuled,
                        localIdentName: localIdentName,
                        namedExport: true,
                        exportOnlyLocals: true
                    },
                },
            },
            {
                loader: "postcss-loader",
                options: Object.assign({
                    postcssOptions: {
                        plugins: [
                            "postcss-preset-env",
                        ],
                    },
                }, ConfigHelper.get("postcss-loader", {})),
            },
        ], extraLoaders);
    }
    getRules(isDebugMode) {
        const rules = [];
        rules.push({
            exclude: /\.d\.ts$/i,
            test: /[\\/]src[\\/]isomorphic[\\/].+[\\/][A-Z][^\\/]+[\\/]index\.tsx?$/,
            loader: "alcor-loaders/connect-default-param-loader",
        });
        rules.push({
            test: /\.css$/i,
            use: this.getCssLoaders(isDebugMode),
            type: "javascript/auto",
        });
        rules.push({
            test: /\.less$/i,
            use: this.getCssLoaders(isDebugMode, [
                {
                    loader: "less-loader",
                    options: Object.assign({
                        sourceMap: isDebugMode,
                    }, ConfigHelper.get("less-loader", {})),
                },
            ]),
            type: "javascript/auto",
        });
        rules.push({
            test: /\.s[ac]ss$/i,
            use: this.getCssLoaders(isDebugMode, [
                {
                    loader: "sass-loader",
                    options: Object.assign({
                        sourceMap: isDebugMode,
                    }, ConfigHelper.get("sass-loader", {})),
                },
            ]),
            type: "javascript/auto",
        });
        rules.push({
            test: /\.(ico|jpg|jpeg|png|gif|eot|otf|webp|svg|ttf|woff|woff2)(\?.*)?$/i,
            type: "asset",
            generator: {
                filename: path.resolve(getGlobalConfig().clientOutput, assetModuleFilename),
            },
        });
        return rules;
    }
    async done() {
        console.log(green(`${cyan(this.taskName)} task completed.\n`));
        this.compileFinishedCallback(async () => {
            if (this.autoRun === true && this.isDebugMode === true) {
                const serverEntry = "index";
                if (timeout) {
                    clearTimeout(timeout);
                }
                timeout = setTimeout(async () => {
                    await RunServer(serverEntry, this.debugPort);
                }, 800);
            }
        });
    }
    setAutoRun(autoRun = true) {
        this.autoRun = autoRun;
        this.debugPort = ConfigHelper.get("debugPort", 0);
        log.info("debugPort", this.debugPort);
        return this;
    }
    async compile() {
        log.info("->", cyan(this.taskName), HelperTask.taking());
        const config = {
            entry: { "index": this.src, },
            module: {
                parser: {
                    javascript: {
                        commonjsMagicComments: true,
                    },
                },
                rules: this.getRules(this.isDebugMode),
            },
            name: this.taskName,
            output: {
                path: this.dist,
            },
            plugins: this.getPlugins(),
        };
        log.info("ServerPack.pack", { config: JSON.stringify(config) });
        await super.compile(config);
    }
    setHotReloadMode(isHotReload) {
        this.isHotReload = isHotReload;
        return this;
    }
}
export default ServerPack;
